@@ -21,8 +21,9 @@
  */
 #define LDNS_BUILD_CONFIG_HAVE_SSL         1
 #define LDNS_BUILD_CONFIG_HAVE_INTTYPES_H  1
-#define LDNS_BUILD_CONFIG_HAVE_ATTR_FORMAT 1
+#define LDNS_BUILD_CONFIG_HAVE_ATTR_FORMAT 0
 #define LDNS_BUILD_CONFIG_HAVE_ATTR_UNUSED 1
+#define LDNS_BUILD_CONFIG_HAVE_SOCKLEN_T   1
 
 /*
  * HAVE_STDBOOL_H is not available when distributed as a library, but no build 
@@ -65,4 +66,8 @@ typedef bool _Bool;
 #define ATTR_UNUSED(x)  x
 #endif /* !LDNS_BUILD_CONFIG_HAVE_ATTR_UNUSED */
 
+#if !LDNS_BUILD_CONFIG_HAVE_SOCKLEN_T
+typedef int socklen_t;
+#endif
+
 #endif /* LDNS_COMMON_H */
